@@ -29,9 +29,11 @@ from logilab.common.deprecation import deprecated, class_renamed
 from logilab.common.decorators import cached
 from logilab.common.registry import yes
 
+from threading import Lock
+
 from cubicweb import neg_role, appobject
 from cubicweb.schema import display_name
-from cubicweb.utils import json_dumps
+from cubicweb.utils import json_dumps, make_uid
 from cubicweb.predicates import non_final_entity, match_kwargs
 from cubicweb.view import EntityView
 from cubicweb.web import uicfg, stdmsgs
@@ -55,9 +57,17 @@ class _DummyForm(object):
 
 rctrl = uicfg.reledit_ctrl
 
+UID = 0
+UIDLOCK = Lock()
+def make_uid():
+    with UIDLOCK:
+        global UID
+        UID += 1
+        return UID
+
 def build_divid(rtype, role, entity_eid):
     """ builds an id for the root div of a reledit widget """
-    return '%s-%s-%s' % (rtype, role, entity_eid)
+    return '%s-%s-%s-%s' % (rtype, role, entity_eid, make_uid())
 
 @cached
 def compute_ttypes(rschema, role):
@@ -84,6 +94,11 @@ class RelatedEntityEdit(EntityView):
             evid = evid or self._default_evid
         extradata = dict((k, kwargs.get(k))
                          for k in self._parent_attrs)
+
+        self.display_entitites(action, reload, evid, extradata, **kwargs)
+
+    def display_entitites(self, action, reload, evid, extradata, **kwargs):
+        w = self.w; rset = self.cw_rset
         with t.div(w, Class=self.__regid__):
             for i in xrange(len(rset)):
                 with t.div(w, Class='%s-item' % self.__regid__):
@@ -124,7 +139,7 @@ class AutoClickAndEditFormView(EntityView):
         rschema = self._cw.vreg.schema[rtype]
         self._rules = rctrl.etype_get(self.entity.e_schema.type, rschema.type, role, '*')
         reload = self._compute_reload(rschema, role, reload)
-        divid = build_divid(rtype, role, self.entity.eid)
+        divid = kwargs.get('divid', build_divid(rtype, role, self.entity.eid))
         if rschema.final:
             self._handle_attribute(rschema, role, divid, reload, action, kwargs)
         else:
@@ -133,7 +148,7 @@ class AutoClickAndEditFormView(EntityView):
             else:
                 self._handle_relation(rschema, role, divid, reload, action, kwargs)
 
-    def _handle_attribute(self, rschema, role, divid, reload, action, kwargs):
+    def _handle_attribute(self, rschema, role, divid, reload, action, extradata):
         rvid = self._rules.get('rvid', None)
         if rvid is not None:
             value = self._cw.view(rvid, entity=self.entity,
@@ -147,7 +162,7 @@ class AutoClickAndEditFormView(EntityView):
         form_handler = self._cw.vreg['reledit'].select('relation-form-handler', self._cw,
                                                        entity=self.entity, rtype=rschema.type)
         form_handler.apply(self.w, value, self.entity, rschema, role, divid, reload, action,
-                           extradata=kwargs)
+                           extradata=extradata)
 
     def _compute_value_editperm(self, rschema, role, rvid, **viewargs):
         related_rset = self.entity.related(rschema.type, role)
@@ -157,17 +172,17 @@ class AutoClickAndEditFormView(EntityView):
             value = self._compute_default_value(rschema, role)
         return value, self._should_edit_relation(rschema, role)
 
-    def _handle_relation(self, rschema, role, divid, reload, action, kwargs):
+    def _handle_relation(self, rschema, role, divid, reload, action, extradata):
         rvid = self._rules.get('rvid', 'autolimited')
-        value, permission = self._compute_value_editperm(rschema, role, rvid)
+        value, permission = self._compute_value_editperm(rschema, role, rvid, **extradata)
         if not permission:
             return self.w(value)
         form_handler = self._cw.vreg['reledit'].select('relation-form-handler', self._cw,
                                                        entity=self.entity, rtype=rschema.type)
         form_handler.apply(self.w, value, self.entity, rschema, role, divid,
-                           reload, action, extradata=kwargs)
+                           reload, action, extradata=extradata)
 
-    def _handle_composite(self, rschema, role, divid, reload, action, kwargs):
+    def _handle_composite(self, rschema, role, divid, reload, action, extradata):
         rvid = self._rules.get('rvid', 'edit-related-entity')
         value, permission = self._compute_value_editperm(rschema, role, rvid,
                                                          reload=reload, action=action,
@@ -179,7 +194,7 @@ class AutoClickAndEditFormView(EntityView):
                                                        entity=self.entity, rtype=rschema.type)
         form_handler.apply(self.w, value, self.entity, rschema, role, divid,
                            reload, action, self._may_add_related(rschema, role), False,
-                           extradata=kwargs)
+                           extradata=extradata)
 
     def _compute_reload(self, rschema, role, reload):
         ctrl_reload = self._rules.get('reload', reload)
@@ -289,13 +304,13 @@ class ReleditFormHandler(appobject.AppObject):
                   view_form_method):
         w = self.w
         with t.div(w, id='%s-reledit' % divid,
-                   onmouseout="jQuery('#%s').addClass('hidden')" % divid,
-                   onmouseover="jQuery('#%s').removeClass('hidden')" % divid,
+                   onmouseout="jQuery('#%s').addClass('invisible')" % divid,
+                   onmouseover="jQuery('#%s').removeClass('invisible')" % divid,
                    Class='releditField'):
-            with t.div(w, id='%s-value' % divid, Class='editableFieldValue'):
+            with t.span(w, id='%s-value' % divid, Class='editableFieldValue'):
                 w(value)
             form.render(renderer=renderer, w=w)
-            with t.div(w, id=divid, Class='editableField hidden'):
+            with t.span(w, id=divid, Class='editableField invisible'):
                 view_form_method()
 
 class ReleditEntityFormHandler(ReleditFormHandler):
@@ -329,12 +344,12 @@ class ReleditEntityFormHandler(ReleditFormHandler):
             event_args.update(extradata)
         return event_args
 
-    def _build_renderer(self, entity):
+    def _build_renderer(self, entity, **formargs):
         return self._cw.vreg['formrenderers'].select(
             self._form_renderer_id, self._cw, entity=entity,
             display_label=True, table_class='attributeForm',
             button_bar_class='buttonbar',
-            display_progress_div=False)
+            display_progress_div=False, **formargs)
 
     def _build_form(self, entity, divid, reload, action, extradata=None, **formargs):
         event_args = self._build_args(entity, reload, action, extradata)
@@ -355,27 +370,27 @@ class ReleditEntityFormHandler(ReleditFormHandler):
         # handle buttons
         self._setup_form_buttons(form, divid)
         form.event_args = event_args
-        return form, self._build_renderer(entity)
+        return form, self._build_renderer(entity, **formargs)
 
     def wrap_form(self, divid, value, form, renderer,
                   _edit_related, _add_related, _delete_related,
                   view_form_method):
         w = self.w
         with t.div(w, id='%s-reledit' % divid,
-                   onmouseout="jQuery('#%s').addClass('hidden')" % divid,
-                   onmouseover="jQuery('#%s').removeClass('hidden')" % divid,
+                   onmouseout="jQuery('#%s').addClass('invisible')" % divid,
+                   onmouseover="jQuery('#%s').removeClass('invisible')" % divid,
                    Class='releditField'):
             with t.div(w, id='%s-value' % divid,
                        Class='editableFieldValue'):
                 w(value)
             form.render(w=w, renderer=renderer)
-            with t.div(w, id=divid, Class='editableField hidden'):
+            with t.div(w, id=divid, Class='editableField invisible'):
                 view_form_method()
 
     def _edit_action(self, divid, args):
         w = self.w
         args['action'] = 'edit-related'
-        with t.div(w, id='%s-update' % divid,
+        with t.span(w, id='%s-update' % divid,
                    onclick=xml_escape(self._onclick % json_dumps(args)),
                    title=self._cw._(self._editzonemsg),
                    Class='editableField'):
@@ -384,7 +399,7 @@ class ReleditEntityFormHandler(ReleditFormHandler):
     def _del_action(self, divid, args):
         w = self.w
         args['action'] = 'delete-related'
-        with t.div(w, id='%s-delete' % divid,
+        with t.span(w, id='%s-delete' % divid,
                    onclick=xml_escape(self._onclick % json_dumps(args)),
                    title=self._cw._(self._deletemsg),
                    Class="editableField"):
@@ -423,8 +438,7 @@ class ReleditRelationFormHandler(ReleditFormHandler):
         return self._addzone % {'msg': xml_escape(self._cw._(self._addmsg)),
                                 'logo': xml_escape(self._cw.data_url(self._addzonelogo))}
 
-    def _build_args(self, entity, rtype, role, reload, action, extradata=None):
-        divid = build_divid(rtype, role, entity.eid)
+    def _build_args(self, entity, rtype, role, reload, action, divid, extradata=None):
         event_args = {'divid' : divid, 'eid' : entity.eid, 'rtype' : rtype,
                       'reload' : reload, 'action': action,
                       'role' : role, 'fname' : u'reledit_form'}
@@ -447,18 +461,19 @@ class ReleditRelationFormHandler(ReleditFormHandler):
         self._cw.form['__linkto'] = '%s:%s:%s' % (rschema, entity.eid, neg_role(role))
         return label, edit_entity
 
-    def _build_renderer(self, related_entity, display_label):
+    def _build_renderer(self, related_entity, display_label, **formargs):
         return self._cw.vreg['formrenderers'].select(
             self._form_renderer_id, self._cw, entity=related_entity,
             display_label=display_label,
             table_class='attributeForm' if display_label else '',
             button_bar_class='buttonbar',
-            display_progress_div=False)
+            display_progress_div=False,
+            **formargs)
 
     def _build_form(self, entity, rschema, role, divid, reload, action,
                     extradata=None, **formargs):
         rtype = rschema.type
-        event_args = self._build_args(entity, rtype, role, reload, action, extradata)
+        event_args = self._build_args(entity, rtype, role, reload, action, divid, extradata)
         if not action:
             form = _DummyForm()
             form.event_args = event_args
@@ -480,7 +495,7 @@ class ReleditRelationFormHandler(ReleditFormHandler):
         if formid == 'base':
             field = form.field_by_name(rtype, role, entity.e_schema)
             form.append_field(field)
-        return form, self._build_renderer(edit_entity, label)
+        return form, self._build_renderer(edit_entity, label, **formargs)
 
     def _add_action(self, divid, args):
         w = self.w
